package client.controller;

import client.view.*;
import client.model.*;
import rmiUtils.*;
import examples.classServer.*;


public class ClientController {
	private ComputeTask model;
	private ClientGUI gui;
	private ClassFileServer classFileServer;

	public ClientController() {
		init();
	}


	private void init() {
		System.setProperty("java.rmi.server.logCalls","true");  //Logs all RMI activity to System.err
		try {
			//Try to get figure out this server's IP address and save it as the RMI server hostname.
			System.setProperty("java.rmi.server.hostname", java.net.InetAddress.getLocalHost().getHostAddress());

			System.setProperty("java.rmi.server.codebase","http://"+System.getProperty("java.rmi.server.hostname")+":"+IRMI_Defs.CLASS_SERVER_PORT_B+"/");
			System.out.println("java.rmi.server.hostname: "+System.getProperty("java.rmi.server.hostname"));
			System.out.println("java.rmi.server.codebase: "+System.getProperty("java.rmi.server.codebase"));
		}
		catch(Exception e) {
			gui.append("Error getting local host address: "+e+"\n");
		}

		// file.separator is "\" in Windows and "/" in Unix/Linux/Mac.
		String sep = System.getProperty("file.separator");
//		System.setProperty("java.security.policy", "bin"+sep+"rmiUtils"+sep+"server.policy");
		System.setProperty("java.security.policy", "rmiUtils"+sep+"server.policy");
		System.out.println("java.security.policy: "+System.getProperty("java.security.policy"));

		String userDir = System.getProperty("user.dir");
		System.out.println("user.dir: "+userDir);
		gui = new ClientGUI(new IModelAdapter() {
			public String connectTo(String remoteHost) {
				return model.connectTo(remoteHost);
			}

			public String calcPi(int numDigits) {
				return model.calcPi(numDigits);
			}
			
			public void quit(){
				try {
					classFileServer.stop();
				}
				catch(Exception e){
					System.err.println("ClientController: Error stopping the class file server: "+e);
				}
				finally {
					System.exit(0);
				}
			}
		});
		try {
			gui.setRemoteHost(java.net.InetAddress.getLocalHost().getHostAddress());
		}
		catch(Exception e) {
			System.err.println("Error getting local address: "+e);
		}
		model = new ComputeTask(new IViewAdapter() {
			public void append(String s){
				gui.append(s);
			}
		});

		try {
			//classFileServer = new ClassFileServer(IRMI_Defs.CLASS_SERVER_PORT_B, userDir+sep+"bin");
			classFileServer = new ClassFileServer(IRMI_Defs.CLASS_SERVER_PORT_B, userDir);
		} 
		catch (java.io.IOException e) {
			System.out.println("Unable to start ClassServer: " +
					e.getMessage());
			e.printStackTrace();
		}

		gui.setVisible(true);
	}


	public static void main(String[] args) {
		new ClientController();
	}
}