package client.model;

import rmiUtils.*;
import util.IVoidLambda;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.*;
import java.math.BigDecimal;
import compute.*;
import client.model.task.*;

public class ComputeTask {  
  
  private IViewAdapter view;
  
  private ICompute comp;
  
  public ComputeTask(IViewAdapter va) {
    view = va;
    if (System.getSecurityManager() == null) {
      view.append("Installing new Security Manager...\n");
      System.setSecurityManager(new SecurityManager());
      view.append("Security Manager = "+System.getSecurityManager());
    }
  }
  
  private ITextAdapter serverTA;
  private ITextAdapter clientTA = new ITextAdapter() {
    public void append(String s) {
      view.append("[Server] "+s);
    }
  };
  
  private IVoidLambda<String> outputCmd = new IVoidLambda<String> (){
	  public void apply(String... strs){
	    for(String s: strs)view.append(s);
	  }
  };

  public String connectTo(String remoteHost) {
    try {
      view.append("Locating registry at "+remoteHost+"...\n");
      //Registry registry = LocateRegistry.getRegistry(remoteHost, IRMI_Defs.REGISTRY_PORT);
      Registry registry = RegistryFactory.getClientRegistry(remoteHost, outputCmd);
      view.append("Found registry: "+registry+"\n");
      comp = (ICompute) registry.lookup(ICompute.BOUND_NAME);
      view.append("Found remote Compute object: "+ comp+"\n");
      ITextAdapter clientTAstub =(ITextAdapter) UnicastRemoteObject.exportObject(clientTA, ITextAdapter.BOUND_PORT_B);
      serverTA = comp.setTextAdapter(clientTAstub);
      view.append("Got text adapter: "+serverTA+"\n");
      serverTA.append("Hello from the client!\n");
    } 
    catch (Exception e) {
      view.append("Exception connecting to "+remoteHost+": "+e+"\n");
      e.printStackTrace();
      return "No connection established!";
    }
    return "Connection to "+remoteHost+" established!";
  }
  
  public String calcPi(int numDigits) {
    String result = "";
    if(null == comp) return "No ICompute object yet!";
    Pi task = new Pi(numDigits);

    try {
      BigDecimal pi = comp.executeTask(task);
      result = pi.toString();
    } 
    catch (Exception e) {
      view.append("\nComputePi exception: "+e+"\n");
      e.printStackTrace();
    }    
    return result;
  }
  
//  public static void main(String args[]) {
//    System.setProperty("java.rmi.server.codebase", "file:///Users/swong/Documents/My%20Classes/Comp202/RMI/Sun%20Example/classes");
//    System.setProperty("java.rmi.server.hostname", "localhost");
//    System.setProperty("java.rmi.server.hostname", "www.photon.cs.rice.edu");
//    System.setProperty("java.security.policy", "client.policy");
//    view.append("Codebase = "+ System.getProperty("java.rmi.server.codebase")); 
//    view.append("Hostname = "+ System.getProperty("java.rmi.server.hostname")); 
//    view.append("Security Policy = "+ System.getProperty("java.security.policy")); 
    
//    String remoteHost = "www.photon.cs.rice.edu";//"localHost";
    
//  }    
}
