package engine.controller;

import rmiUtils.*;
import compute.*;
import engine.model.*;
import engine.view.*;
import examples.classServer.*;
import util.*;

import java.rmi.registry.*;
import java.rmi.server.*;


public class EngineController {
  
  private EngineGUI gui;
  
  
  public EngineController() {
    init();
  }
  
  private Registry registry;
  private ClassFileServer classFileServer;
  
  private void init() {
    gui = new EngineGUI(new IModelAdapter(){
    	public void quit(){
    		try{
    			registry.unbind(ICompute.BOUND_NAME);
    			System.out.println("EngineController: "+ICompute.BOUND_NAME+" has been unbound.");
    			classFileServer.stop();
    		}
    		catch(Exception e){
    			System.err.println("EngineController: Error unbinding "+ICompute.BOUND_NAME+" or in stopping the class file server:\n"+e);
    		}
			finally {
				System.exit(0);
			}
    	}
    });
    gui.setVisible(true);
    System.setProperty("java.rmi.server.logCalls","true");  //Logs all RMI activity to System.err
    //Set the RMI server codebase to the working directory
    //System.setProperty("java.rmi.server.codebase",convertPathToURL(System.getProperty("user.dir")));

    try {
      //Try to get figure out this server's IP address and save it as the RMI server hostname.
      System.setProperty("java.rmi.server.hostname", java.net.InetAddress.getLocalHost().getHostAddress());
      
      System.setProperty("java.rmi.server.codebase","http://"+System.getProperty("java.rmi.server.hostname")+":"+IRMI_Defs.CLASS_SERVER_PORT_A+"/");
     
      System.out.println("java.rmi.server.hostname: "+System.getProperty("java.rmi.server.hostname"));
      System.out.println("java.rmi.server.codebase: "+System.getProperty("java.rmi.server.codebase"));
     
    }
    catch(Exception e) {
      gui.append("Error getting local host address: "+e+"\n");
    }
    
    // The server security policy is in the rmiUtils directory
	// file.separator is "\" in Windows and "/" in Unix/Linux/Mac.
	String sep = System.getProperty("file.separator");
	//System.setProperty("java.security.policy", "bin"+sep+"rmiUtils"+sep+"server.policy");
	System.setProperty("java.security.policy", "rmiUtils"+sep+"server.policy");
    
    outputCmd.apply("Codebase = "+ System.getProperty("java.rmi.server.codebase")+"\n"); 
    outputCmd.apply("Hostname = "+ System.getProperty("java.rmi.server.hostname")+"\n"); 
    outputCmd.apply("Security Policy = "+ System.getProperty("java.security.policy")+"\n"); 
    
    if (System.getSecurityManager() == null) {
    	outputCmd.apply("Installing new SecurityManager..."+"\n");
        System.setSecurityManager(new SecurityManager());
    }
    outputCmd.apply("Security Manager = "+System.getSecurityManager()+"\n" );
    try {
     // String name = "Compute";
      ICompute engine = new ComputeEngine(new IViewAdapter() {
        public void append(String s) {
        	outputCmd.apply(s);
        }
      });
      outputCmd.apply("Instantiated new Compute engine: "+engine+"\n");
      // Use this technique rather than the simpler "registry.rebind(name, engine);"
      // because it enables us to specify a port number so we can open that port on the firewall
      ICompute stub =(ICompute) UnicastRemoteObject.exportObject(engine, ICompute.BOUND_PORT); // ITextAdapter.BOUND_PORT_A);
      outputCmd.apply("Looking for registry..."+"\n");
      registry = RegistryFactory.getServerRegistry(outputCmd);

      outputCmd.apply("Found registry: "+ registry+ "\n");
      registry.rebind(ICompute.BOUND_NAME, stub);
      outputCmd.apply("ComputeEngine bound to "+ICompute.BOUND_NAME+"\n");
    } 
    catch (Exception e) {
      System.err.println("ComputeEngine exception:"+"\n");
      e.printStackTrace();
      System.exit(-1);
    }
    
    String userDir = System.getProperty("user.dir");
    System.out.println("user.dir: "+userDir);
    try {
      classFileServer = new ClassFileServer(IRMI_Defs.CLASS_SERVER_PORT_A, System.getProperty("user.dir"));
    } 
    catch (java.io.IOException e) {
      outputCmd.apply("Unable to start ClassServer: " +
                         e.getMessage());
      e.printStackTrace();
    }

    outputCmd.apply("Waiting..."+"\n");
  }
  
  private IVoidLambda<String> outputCmd = new IVoidLambda<String> (){
	  public void apply(String... strs){
	    for(String s: strs)gui.append(s);
	  }
  };
  
  public static void main(String[] args) {
    new EngineController();
  }
  
  
  public static String convertPathToURL(String path) {
    String result="";
    try {
      // Need to convert via URI so that non-standard characters, e.g. spaces are properly handled
      java.net.URL url = (new java.io.File(path)).toURI().toURL();
      result = url.toString();
    }
    catch(Exception e) {
      System.err.println("Error converting \""+path+"\" to URL: "+e);
    }
    return result;
  }
}
