package engine.model;

import compute.*;
import java.rmi.server.*;

/**
 * A concrete implementation of an ICompute engine
 */
public class ComputeEngine implements ICompute {
  
  /**
   * The adapter to the view
   */
  private IViewAdapter viewAdapter;
  
  /**
   * The constructor for the class
   * @param va The adapter to the view
   */
  public ComputeEngine(IViewAdapter va) {
    super();
    viewAdapter = va;
  }
  
  private ITaskViewAdapter taskView = new ITaskViewAdapter(){
    public void append(String s) {
      viewAdapter.append(s);
    }
  };
  /**
   * Executes the given task and returns the result
   * @param <T> The type of the return value from executing the given ITask object
   * @param t The ITask object to execute
   * @return The result from executing the ITask object
   */
  public <T> T executeTask(ITask<T> t) {
    try {
      t.setTaskViewAdapter(taskView);
      viewAdapter.append("Executing task: "+t+"\n");
      T result = t.execute();
      viewAdapter.append("Task result = "+result+"\n");
      return result;
    }
    catch(Exception e) {
      viewAdapter.append("executeTask exception: "+e);
      return null;
    }
  }
  private ITextAdapter clientTA;
  
  public ITextAdapter setTextAdapter(ITextAdapter ta) {
    clientTA = ta;
    viewAdapter.append("Got text adapter: "+clientTA+"\n");
    ITextAdapter myTA = new ITextAdapter() {
      public void append(String s) {
        viewAdapter.append("[Client] "+s);
      }
    };
    try {
      clientTA.append("Hello from the server!\n");
      ITextAdapter taStub =(ITextAdapter) UnicastRemoteObject.exportObject(myTA, ITextAdapter.BOUND_PORT_A);
      return taStub;
    }
    catch(Exception e) {
      System.err.println("ComputeEngine.setClient: "+e);
      return null;
    }
  }

  
  
}
