package engine.view;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * This class represents the "view" for the Model-View-Controller architecture used by the Hangman game system. 
 * The GUI provides all the input and output user interfaces needed by the user to play the game. 
 * The communication to the the model is accomplished via two adapters.
 */
public class EngineGUI extends JFrame {


	/**
	 * 
	 */
	private static final long serialVersionUID = -4584473405991544203L;

	private JPanel controlPanel;

	private JTextArea outputTA;

	private JButton quitBtn;

	private IModelAdapter model;
	
	public EngineGUI(IModelAdapter model){
		super("RMI Engine");
		this.model = model;
		initGUI();
	}

	/**Component initialization*/
	protected void initGUI() {
		//setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		this.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				quit();
			}
		});
		controlPanel = new JPanel();

		outputTA = new JTextArea();
		quitBtn = new JButton("Quit");

		setSize(800, 300);
		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(controlPanel, BorderLayout.NORTH);
		JScrollPane scroll = new JScrollPane();
		scroll.setAutoscrolls(true);
		scroll.getViewport().add(outputTA, null);
		contentPane.add(scroll, BorderLayout.CENTER);

		controlPanel.add(quitBtn);

		quitBtn.addActionListener(new java.awt.event.ActionListener() { 
			public void actionPerformed(ActionEvent e) { 
				quit();
			} 
		}); 
	}

	/**
	 * Centralized management point for exiting the application.
	 * All calls to exit the system should go through here.
	 */
	private void quit() {
		System.out.println("EngineGUI: Server is quitting...");
		model.quit();
		//System.exit(0);
	}

	/**
	 * Appends the given string to the end of the output text area.
	 * Will force the the scroll pane to show the appended text.
	 * Does NOT add a linefeed to the string!
	 * @param s The string to append
	 */
	public void append(String s) {
		outputTA.append(s);
		//Force the JScrollPane to go to scroll down to the new text
		outputTA.setCaretPosition(outputTA.getText().length()); 
	}
}