package rmiUtils;

import java.rmi.*;
import java.rmi.registry.*;
import util.IVoidLambda;

/**
 * Utilities to position of the beer 
 * @author swong
 *
 */
public class RegistryFactory {
	public final static IVoidLambda<String> StdErrVoidLambda = new IVoidLambda<String>(){
		public void apply(String... msgs){
			for(String s: msgs) {
				System.err.println(s);
			}
		}
	};
	
	/**
	 * Takes multiple output lambdas and creates a single lambda to be used to call all of them.
	 * @param outputs  vararg collection of output lambdas.
	 * @return
	 */
	public static IVoidLambda<String> makeOutputLambda(final IVoidLambda<String>... outputs){
		return new IVoidLambda<String>(){
			public void apply(String... strs){
				for(IVoidLambda<String> cmd: outputs){
					for(String s: strs) cmd.apply(s);
				}
			}
		};
	}
	
	public static final Registry getServerRegistry(IVoidLambda<String>... outputs){
		IVoidLambda<String> output = outputs.length==0? 
				makeOutputLambda(StdErrVoidLambda): makeOutputLambda(outputs);

		try {
			// First try to create a registry
			return LocateRegistry.createRegistry(IRMI_Defs.REGISTRY_PORT);	
		}
		catch(RemoteException e){
			output.apply("Could not create registry: "+e+"\n",
					"Attempting to locate existing registry...");
			try {
				// Creating a new registry will fail if it already exists, so try to find it.
				return LocateRegistry.getRegistry(IRMI_Defs.REGISTRY_PORT);
			}
			catch(RemoteException e2){
				output.apply("RegistryFactory.getServerRegistry: Could not get registry: \n"+e2);
				System.exit(-1);  // exit the program.
			}		
		}
		return null;  // code should never get here.
	}

	public static final Registry getClientRegistry(String host, IVoidLambda<String>... outputs){
		IVoidLambda<String> output = outputs.length==0? 
				makeOutputLambda(StdErrVoidLambda): makeOutputLambda(outputs);
		try{
			return LocateRegistry.getRegistry(host, IRMI_Defs.REGISTRY_PORT);
		}
		catch(RemoteException e){
			output.apply("RegistryFactory.getClientRegistry: Could not get registry: \n"+e);
			System.exit(-1);  // exit the program.
		}
		return null;  // code should never get here.
	}
}
